/* This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef FVWMNETHINTS_H
#define FVWMNETHINTS_H

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xproto.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/cursorfont.h>


#include <libs/fvwmlib.h>
#include <fvwm/fvwm.h>
#include <libs/vpacket.h>
#include "my_imlib.h"

typedef struct strut
{
  int left;
  int right;
  int top;
  int bottom;
} strut;

typedef struct strut_max
{
  int horiz;
  int vert;
  int h;
  int v;
  int x;
  int y;
} strut_max;

typedef struct
{
  unsigned do_skip_pager_window_list : 1;
  unsigned do_has_strut : 1;
  unsigned is_net_modal : 1;
  unsigned is_net_stays_on_top : 1;
  unsigned net_mini_icon_state : 2;
  unsigned net_icon_state : 2;
#define NO_NET_ICON        0
#define NET_ICON           1
#define SIZE_NET_ICON      2
#define FVWM_NET_ICON      3
  unsigned net_mini_icon_mode : 2;
  unsigned net_icon_mode : 2;
  unsigned has_net_wm_name : 1;
  unsigned has_net_wm_icon_name : 1;
} my_window_flags;

typedef struct
{
  unsigned is_sticky : 1;
  unsigned do_circulate_skip : 1;
  unsigned do_window_list_skip : 1;
  unsigned never_focus : 1;
  unsigned has_border : 1;
  unsigned has_title : 1;
  unsigned force_decoration : 1;
  int layer;
} net_style;

typedef struct net_window
{
  Window w;                   /* the child window */
  Window frame;
  Window icon_w;
  Pixmap icon_pixmap_w;
  int x;                      /* "true" size and position (relative and
				 frame is used) */
  int y;
  int width;
  int height;

  strut frame_strut;         /* for _KDE_NET_WM_FRAME_STRUT */
  int desk;
  int default_layer;
  int layer;
  char *icon_name;
  char *window_name;
  char *res_name;
  char *res_class;
  unsigned char *net_wm_name;
  unsigned char *net_wm_icon_name;
  int net_wm_visible_name_count;
  int net_wm_icon_visible_name_count;
  char *mini_icon_file;       /* to have the ability to reload icons */
  Picture mini_icon;
#if HAVE_AN_IMLIB
  myImlibImage *Imlib_mini;
#endif
  char *icon_file;
  window_flags flags;
  my_window_flags my_flags;   /* stuff not yet supported by FVWM */
  Atom net_window_type;
  strut strut_w;
  strut dyn_strut_w;
  strut_max s_max;
  struct net_window *next;     /* next  window */
} NetWindow;

#define DO_SKIP_PAGER_WINDOW_LIST(fw) ((fw)->my_flags.do_skip_pager_window_list)
#define SET_DO_SKIP_PAGER_WINDOW_LIST(fw,x) \
          (fw)->my_flags.do_skip_pager_window_list = !!(x)
#define DO_HAS_STRUT(fw) ((fw)->my_flags.do_has_strut)
#define SET_DO_HAS_STRUT(fw,x) (fw)->my_flags.do_has_strut = !!(x)

#define IS_NET_MODAL(fw) ((fw)->my_flags.is_net_modal)
#define SET_IS_NET_MODAL(fw,x) (fw)->my_flags.is_net_modal = !!(x)
#define IS_NET_STAYS_ON_TOP(fw) ((fw)->my_flags.is_net_stays_on_top)
#define SET_IS_NET_STAYS_ON_TOP(fw,x) (fw)->my_flags.is_net_stays_on_top = !!(x)

#define SET_NET_MINI_ICON_STATE(fw,x) (fw)->my_flags.net_mini_icon_state = x
#define DO_HAS_FVWM_NET_MINI_ICON(fw) ((fw)->my_flags.net_mini_icon_state == FVWM_NET_ICON)
#define DO_HAS_SIZE_NET_MINI_ICON(fw) ((fw)->my_flags.net_mini_icon_state == SIZE_NET_ICON)
#define DO_HAS_NET_MINI_ICON(fw) ((fw)->my_flags.net_mini_icon_state == NET_ICON)
#define DO_HAS_NO_NET_MINI_ICON(fw) ((fw)->my_flags.net_mini_icon_state == NO_NET_ICON)

#define SET_NET_ICON_STATE(fw,x) (fw)->my_flags.net_icon_state = x
#define DO_HAS_FVWM_NET_ICON(fw) ((fw)->my_flags.net_icon_state == FVWM_NET_ICON)
#define DO_HAS_SIZE_NET_ICON(fw) ((fw)->my_flags.net_icon_state == SIZE_NET_ICON)
#define DO_HAS_NET_ICON(fw) ((fw)->my_flags.net_icon_state == NET_ICON)
#define DO_HAS_NO_NET_ICON(fw) ((fw)->my_flags.net_icon_state == NO_NET_ICON)

#define SET_NET_MINI_ICON_OVERRIDE_MODE(fw,x) (fw)->my_flags.net_mini_icon_mode = x
#define DO_OVERRIDE_NET_MINI_ICON(fw) ((fw)->my_flags.net_mini_icon_mode ==  FVWM_NET_SET_ICON_MODE_OVERRIDE)
#define DO_SIZE_OVERRIDE_NET_MINI_ICON(fw) ((fw)->my_flags.net_mini_icon_mode == FVWM_NET_SET_ICON_MODE_SIZE_OVERRIDE)
#define DO_ADD_NET_MINI_ICON(fw) ((fw)->my_flags.net_mini_icon_mode == FVWM_NET_SET_ICON_MODE_ADD)
#define DO_NOT_ADD_NET_MINI_ICON(fw) ((fw)->my_flags.net_mini_icon_mode == FVWM_NET_SET_ICON_MODE_NO)

#define SET_NET_ICON_OVERRIDE_MODE(fw,x) (fw)->my_flags.net_icon_mode = x
#define DO_OVERRIDE_NET_ICON(fw) ((fw)->my_flags.net_icon_mode ==  FVWM_NET_SET_ICON_MODE_OVERRIDE)
#define DO_SIZE_OVERRIDE_NET_ICON(fw) ((fw)->my_flags.net_icon_mode == FVWM_NET_SET_ICON_MODE_SIZE_OVERRIDE)
#define DO_ADD_NET_ICON(fw) ((fw)->my_flags.net_icon_mode == FVWM_NET_SET_ICON_MODE_ADD)
#define DO_NOT_ADD_NET_ICON(fw) ((fw)->my_flags.net_icon_mode == FVWM_NET_SET_ICON_MODE_NO)
#define SET_HAS_NET_WM_NAME(fw,x)  (fw)->my_flags.has_net_wm_name = x
#define DO_HAS_NET_WM_NAME(fw)  ((fw)->my_flags.has_net_wm_name)
#define SET_HAS_NET_WM_ICON_NAME(fw,x)  (fw)->my_flags.has_net_wm_icon_name = x
#define DO_HAS_NET_WM_ICON_NAME(fw)  ((fw)->my_flags.has_net_wm_icon_name)

typedef struct ScreenInfo
{
  unsigned long screen;
  int MyDisplayWidth;	/* my copy of DisplayWidth(dpy, screen) */
  int MyDisplayHeight;  /* my copy of DisplayHeight(dpy, screen) */

  Window Root;
  Window Net_w;

  int TopLayer;
  int DefaultLayer;
  int BottomLayer;

  int VxMax;             /* Max location for top left of virt desk */
  int VyMax;
  int VxPages;		 /* desktop size */
  int VyPages;
  int VWidth;		 /* Size of virtual desktop */
  int VHeight;
  int Vx;                /* Current loc for top left of virt desk */
  int Vy;
  int CurrentDesk;

  strut my_strut;

} ScreenInfo;

/* Motif  window hints structure */
typedef struct
{
  CARD32      flags;
  CARD32      functions;
  CARD32      decorations;
  INT32       inputMode;
} PropMotifWmHints;

typedef PropMotifWmHints        PropMwmHints;

/* Motif window hints */
#define MWM_HINTS_FUNCTIONS     (1L << 0)
#define MWM_HINTS_DECORATIONS   (1L << 1)
#define MWM_HINTS_INPUT_MODE    (1L << 2)

/* bit definitions for MwmHints.functions */
#define MWM_FUNC_ALL            (1L << 0)
#define MWM_FUNC_RESIZE         (1L << 1)
#define MWM_FUNC_MOVE           (1L << 2)
#define MWM_FUNC_MINIMIZE       (1L << 3)
#define MWM_FUNC_MAXIMIZE       (1L << 4)
#define MWM_FUNC_CLOSE          (1L << 5)

/* values for MwmHints.input_mode */
#define MWM_INPUT_MODELESS                      0
#define MWM_INPUT_PRIMARY_APPLICATION_MODAL     1
#define MWM_INPUT_SYSTEM_MODAL                  2
#define MWM_INPUT_FULL_APPLICATION_MODAL        3

/* bit definitions for MwmHints.decorations */
#define MWM_DECOR_ALL                 (1L << 0)
#define MWM_DECOR_BORDER              (1L << 1)
#define MWM_DECOR_RESIZEH             (1L << 2)
#define MWM_DECOR_TITLE               (1L << 3)
#define MWM_DECOR_MENU                (1L << 4)
#define MWM_DECOR_MINIMIZE            (1L << 5)
#define MWM_DECOR_MAXIMIZE            (1L << 6)

#define PROP_MOTIF_WM_HINTS_ELEMENTS  4
#define PROP_MWM_HINTS_ELEMENTS       PROP_MOTIF_WM_HINTS_ELEMENTS

/* extern function */
void SetAlarm();

NetWindow *GetNetWindow(Window w);
void UpdateNetModal(Window w, Bool state);
void UpdateStrutMaximize(Window w, int horiz, int vert, int h, int v,
			 int x, int y);
void UpdateSkipPager(Window w, Bool state);
#if HAVE_AN_IMLIB
void SendMiniIcon(Window w);
Bool BuildMiniIcon(NetWindow *t, char *path, Bool rebuild);
#endif

#endif

