/* Copyright 2001, Olivier Chapuis. No guarantees or warantees or anything
 * are provided or implied in any way whatsoever. Use this program at your
 * own risk. Permission to use this program for any purpose is given,
 * as long as the copyright is kept intact. */

/* This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>

#include <libs/fvwmlib.h>
#include "libs/Module.h"
#include "fvwm/fvwm.h"

#include "FvwmNetHints.h"
#include "net.h"
#include "parse.h"
#include "windowlist.h"
#include "Mallocs.h"
#include "net_debug.h"
#include "update.h"

#define FNH_DESKTOP_START_STOP_PREFIX ".fnh-desktop"
#define FNH_START_STOP_PREFIX ".fnh"

/* name of the persitent config file */
#define FNH_PERSITENT_CONFIG_FILE ".fnh-pconfig"

/* do we use the persistent config for the number of desktops? */
Bool UsePNumberOfDesktops = False;

extern char *MyName;
extern int fd[2];

char *FnhSysStartStopFile = NULL;
char *FnhSysDesktopStartStopFile = NULL;

extern ScreenInfo Scr;
extern Display *dpy;
extern NetWindow *Start;
extern NetWindow *FocusWin;
extern int Started;

/* Config */
extern char *ImagePath;
extern long NumberOfDesktops;
extern char *NetAnimCommand;
extern char *NetAnimWindowName;
#if HAVE_ICONV
extern char *IconvCharset;
#endif

extern char *FnhSysStartFile;
extern char *FnhSysStopFile;
extern char *FnhSysStartDesktopFile;
extern char *FnhSysStopDesktopFile;

/* Next: Todo ! */
extern Bool PagerSkipListUseSkipList;
extern Bool SkipListUsePagerSkipList;
extern Bool ForceSkipList;
extern Bool ForcePagerList;
extern Bool KsmServerWorkAround;

extern Bool StrutOverride;
extern Bool UseStrutMaximize;
extern Bool UseDynamicStrutMaximize;
extern unsigned DefaultHorizMaximize;
extern unsigned DefaultVertMaximize;

extern Bool UseStrutPlacement;
extern Bool UseExtendedName;

extern Bool AcceptDesktopGeometry;
extern Bool AcceptStaysOnTop;
extern int StaysOnTopLayer;
extern int StaysPutLayer;

/* Next: Todo ! */
extern net_style DockNetStyle;
extern net_style ToolBarNetStyle;
extern net_style MenuNetStyle;
extern net_style DialogNetStyle;
extern net_style DesktopNetStyle;
/* this not a WM_TYPE it is WM_STATE ! */
extern net_style ModalNetStyle;

extern char *NetActivateWindow;
extern char *NetCloseWindow;

extern unsigned DoSendMiniIcon;
extern int MiniIconSize[2];
extern unsigned MiniIconOverrideMode;
extern unsigned IconOverrideMode;
extern unsigned MiniIconMaxSize;

extern Atom ATOM_NET_WM_WINDOW_TYPE_DESKTOP;
extern Atom ATOM_NET_WM_WINDOW_TYPE_DOCK;
extern Atom ATOM_NET_WM_WINDOW_TYPE_TOOLBAR;
extern Atom ATOM_NET_WM_WINDOW_TYPE_MENU;
extern Atom ATOM_NET_WM_WINDOW_TYPE_DIALOG;

/* intern functions */
void SetNetStyle(net_style *style,net_style st);
void ParseNetStyle(char *line,net_style *st);

/* ***************************************************************************
 * 
 * *************************************************************************** */
void ParseConfig(void)
{
  char *tline = NULL;
  net_style st;
  
  /* First set the default */
  NumberOfDesktops = DEFAULT_NUMBER_OF_DESKTOPS;
  AcceptDesktopGeometry = False;

  AcceptStaysOnTop = True;
  StaysOnTopLayer = DEFAULT_TOP_LAYER;
  StaysPutLayer = DEFAULT_DEFAULT_LAYER;

  st.is_sticky = 1;
  st.do_circulate_skip = 1;
  st.do_window_list_skip = 1;
  st.never_focus = 1;
  st.has_border = 1;
  st.has_title = 0;
  st.layer = -1;
  st.force_decoration = 0;
  SetNetStyle(&DockNetStyle,st);
  SetNetStyle(&ToolBarNetStyle,st);
  st.layer = DEFAULT_TOP_LAYER;
  st.has_border = 0;
  SetNetStyle(&MenuNetStyle,st);
  st.layer = LAYER_NET_WM_WINDOW_TYPE_DESKTOP;
  SetNetStyle(&DesktopNetStyle,st);
  st.is_sticky = 0;
  st.do_circulate_skip = 0;
  st.do_window_list_skip = 0;
  st.never_focus = 0;
  st.has_border = 1;
  st.has_title = 1;
  st.layer = -1;
  SetNetStyle(&DialogNetStyle,st);
  st.is_sticky = 1;
  st.layer = DEFAULT_TOP_LAYER;
  SetNetStyle(&ModalNetStyle,st); /* this not a WM_TYPE it is WM_STATE ! */

  CopyString(&NetActivateWindow,DEFAULT_NET_ACTIVATE_WINDOW_CMD);
  CopyString(&NetCloseWindow,DEFAULT_NET_CLOSE_WINDOW_CMD);
  
  UseStrutMaximize = True;
  UseDynamicStrutMaximize = True;
  UseStrutPlacement = True;
  UseExtendedName = False;
  StrutOverride = False;
  DefaultHorizMaximize = 100;
  DefaultVertMaximize = 100;

  DoSendMiniIcon = DO_SEND_NET_ICON;
  MiniIconSize[0] = 16;
  MiniIconSize[1] = 16;
  MiniIconOverrideMode = FVWM_NET_SET_ICON_MODE_ADD;
  IconOverrideMode = FVWM_NET_SET_ICON_MODE_ADD;
  MiniIconMaxSize = 22;

  KsmServerWorkAround = True;

  InitGetConfigLine(fd,CatString3("*",MyName,0));
  for (GetConfigLine(fd,&tline); tline != NULL;GetConfigLine(fd,&tline))
  {
    HandleConfig(tline);
  }
}

/* ***************************************************************************
 *
 * *************************************************************************** */
void HandleConfig(char *tline)
{
  net_style st;
  char *resource;
  char *arg1;
  char *tline2;
  char *token;
  char *next;
  Bool update = False;

  arg1 = NULL;
  token = PeekToken(tline, &next);
  
  /* FVWM Config */
  if (StrEquals(token, "DesktopSize"))
  {
    int val[2] = {3,3};

    if (GetIntegerArguments(next, NULL, val, 2) == 2 ||
	GetRectangleArguments(next, &val[0], &val[1]) == 2)
    {
      Scr.VxMax = val[0] * Scr.MyDisplayWidth - Scr.MyDisplayWidth;
      Scr.VyMax = val[1] * Scr.MyDisplayHeight - Scr.MyDisplayHeight;
      if(Scr.VxMax <0)
	Scr.VxMax = 0;
      if(Scr.VyMax <0)
	Scr.VyMax = 0;
      Scr.VxPages = Scr.VxMax / Scr.MyDisplayWidth;
      Scr.VyPages = Scr.VyMax / Scr.MyDisplayHeight;
      Scr.VWidth = Scr.VxMax + Scr.MyDisplayWidth;
      Scr.VHeight = Scr.VyMax + Scr.MyDisplayHeight;
    }
    return;
  }
  else if (StrEquals(token, "ImagePath"))
  {
    if (ImagePath != NULL)
    {
      free(ImagePath);
      ImagePath = NULL;
    }
    GetNextToken(next, &ImagePath);
    return;
  }
  /* Not send by fvwm2 */
  else if (StrEquals(token, "DefaultLayers"))
  {
    int val[3];

    if (GetIntegerArguments(next, NULL, val, 3) == 3)
    {
      Scr.BottomLayer = val[0];
      Scr.DefaultLayer = val[1];
      Scr.TopLayer = val[2];
    }
  }

  /* *********** Module Config ****************** */
  tline2 = GetModuleResource(tline, &resource, MyName);
  if (!resource)
    return;
  if (StrEquals(resource,"DockNetStyle" )) {
    SetNetStyle(&st,DockNetStyle);
    ParseNetStyle(tline2,&st);
    SetNetStyle(&DockNetStyle,st);
    if (Started)
      UpdateNetStyle(ATOM_NET_WM_WINDOW_TYPE_DOCK);
  }
  else if(StrEquals(resource,"ToolBarNetStyle" )) {
    SetNetStyle(&st,ToolBarNetStyle);
    ParseNetStyle(tline2,&st);
    SetNetStyle(&ToolBarNetStyle,st);
    if (Started)
      UpdateNetStyle(ATOM_NET_WM_WINDOW_TYPE_TOOLBAR);
  }
  else if(StrEquals(resource,"MenuNetStyle" )) {
    SetNetStyle(&st,MenuNetStyle);
    ParseNetStyle(tline2,&st);
    SetNetStyle(&MenuNetStyle,st);
    if (Started)
      UpdateNetStyle(ATOM_NET_WM_WINDOW_TYPE_MENU);
  }
  else if(StrEquals(resource,"DesktopNetStyle" )) {
    SetNetStyle(&st,DesktopNetStyle);
    ParseNetStyle(tline2,&st);
    SetNetStyle(&DesktopNetStyle,st);
    if (Started)
      UpdateNetStyle(ATOM_NET_WM_WINDOW_TYPE_DESKTOP);
  }
  else if(StrEquals(resource,"DialogNetStyle" )) {
    SetNetStyle(&st,DialogNetStyle);
    ParseNetStyle(tline2,&st);
    SetNetStyle(&DialogNetStyle,st);
    if (Started)
      UpdateNetStyle(ATOM_NET_WM_WINDOW_TYPE_DIALOG);
  }
  else if(StrEquals(resource,"ModalNetStyle" )) {
    SetNetStyle(&st,ModalNetStyle);
    ParseNetStyle(tline2,&st);
    SetNetStyle(&ModalNetStyle,st);
    if (Started)
      UpdateModalNetStyle();
  }
  else if(StrEquals(resource,"NumberOfDesktops" )) {
    int val[1] = {-1};
    if (GetIntegerArguments(tline2, NULL, val, 1) == 1) {
      if (val[0] > 0 && val[0] <= MAX_NUMBER_OF_DESKTOPS) {
	NumberOfDesktops = val[0];
	if (Started)
	  NET_SetNumberOfDesktops();
      }
      else
	fprintf(stderr,"%s: Argument to NumberOfDesktops should be an\n\
              integer between 1 and %i\n",MyName,MAX_NUMBER_OF_DESKTOPS);
    }
    else
      fprintf(stderr,"%s: Argument to NumberOfDesktops should be an\n\
              integer between 1 and %i\n",MyName,MAX_NUMBER_OF_DESKTOPS);
  }
  else if  (StrEquals(resource,"UsePersistentNumberOfDesktops")) {
    tline2 = GetNextToken(tline2, &arg1);
    if (StrEquals(arg1,"True"))
      UsePNumberOfDesktops = True;
    else if (StrEquals(arg1,"False"))
      UsePNumberOfDesktops = False;
    else
      fprintf(stderr,"%s: wrong argument to UsePersistentNumberOfDesktops\n",
	      MyName);
  }
  else if (StrEquals(resource,"NoIconAction")) {
    if (NetAnimCommand != NULL)
      free(NetAnimCommand);
    if (tline2 != NULL && strlen(tline2) > 0)
      CopyString(&NetAnimCommand,tline2);
    else
      NetAnimCommand = NULL;
  }
  else if (StrEquals(resource,"NoIconWindowName")) {
    if (NetAnimWindowName != NULL)
      free(NetAnimWindowName);
    if (tline2 != NULL && strlen(tline2) > 0)
      CopyString(&NetAnimWindowName,tline2);
    else
       NetAnimWindowName = NULL;
  }
#if HAVE_ICONV
  else if (StrEquals(resource,"Charset")) {
    if (IconvCharset != NULL)
      free(IconvCharset);
    if (tline2 != NULL && strlen(tline2) > 0)
      CopyString(&IconvCharset,tline2);
    else
      IconvCharset = NULL;
    if (Started) {
      if (UseExtendedName && IconvCharset != NULL)
	UpdateVisibleNames(False);
      else
	UpdateVisibleNames(True);
    }
  }
  else if  (StrEquals(resource,"UseExtendedName")) {
    tline2 = GetNextToken(tline2, &arg1);
    if (arg1 == NULL || strlen(arg1) < 1) {
      if (UseExtendedName)
	update = True;
      UseExtendedName = False;
    }
    else if (StrEquals(arg1, "True")) {
      if (!UseExtendedName)
	update = True;
      UseExtendedName = True;
    }
    else if (StrEquals(arg1, "False")) {
      if (UseExtendedName)
	update = True;
      UseExtendedName = False;
    }
    else
      fprintf(stderr,"%s: wrong argument to UseExtendedName\n",MyName);
    if (update && Started) {
	if (UseExtendedName && IconvCharset != NULL)
	  UpdateVisibleNames(False);
	else
	  UpdateVisibleNames(True);
    }
  }
#endif
  else if (StrEquals(resource,"SystemStartStopFiles")) {
    if (FnhSysStartStopFile != NULL)
      free(FnhSysStartStopFile);
    if (tline2 != NULL && strlen(tline2) > 0)
      CopyString(&FnhSysStartStopFile,tline2);
    else
       FnhSysStartStopFile = NULL;
  }
  else if (StrEquals(resource,"SystemDesktopStartStopFiles")) {
    if (FnhSysDesktopStartStopFile != NULL)
      free(FnhSysDesktopStartStopFile);
    if (tline2 != NULL && strlen(tline2) > 0)
      CopyString(&FnhSysDesktopStartStopFile,tline2);
    else
       FnhSysDesktopStartStopFile = NULL;
  }
  else if  (StrEquals(resource,"StaysPutLayer")) {
    int val[1] = {0};
    if (GetIntegerArguments(tline2, NULL, val, 1) == 1 && val[0] > 0) {
       StaysPutLayer = val[0];
    }
    else
      fprintf(stderr,"%s: wrong argument to StaysPutLayer\n",MyName);
  }
  else if  (StrEquals(resource,"StaysOnTopLayer")) {
    int val[1] = {0};
    if (GetIntegerArguments(tline2, NULL, val, 1) == 1 && val[0] > 0) {
       StaysOnTopLayer = val[0];
    }
    else
      fprintf(stderr,"%s: wrong argument to StaysOnTopLayer\n",MyName);
  }
  else if  (StrEquals(resource,"AcceptStaysOnTop")) {
    tline2 = GetNextToken(tline2, &arg1);
    if (arg1 == NULL || strlen(arg1) < 1) {
      if (AcceptStaysOnTop != True) {
	update = True;
	AcceptStaysOnTop = True;
      }
    }
    else if (StrEquals(arg1, "True")) {
      if (AcceptStaysOnTop != True) {
	update = True;
	AcceptStaysOnTop = True;
      }
    }
    else if (StrEquals(arg1, "False")) {
      if (AcceptStaysOnTop != False) {
	update = True;
	AcceptStaysOnTop = False;
      }
    }
    else
      fprintf(stderr,"%s: wrong argument to AcceptStaysOnTop\n",MyName);
    if (Started && update)
      UpdateStaysOnTop(AcceptStaysOnTop);
  }
  else if  (StrEquals(resource,"AcceptDesktopGeometry")) {
    tline2 = GetNextToken(tline2, &arg1);
    if (arg1 == NULL || strlen(arg1) < 1)
      AcceptDesktopGeometry = False;
    else if (StrEquals(arg1, "True"))
      AcceptDesktopGeometry = True;
    else if (StrEquals(arg1, "False"))
      AcceptDesktopGeometry = False;
    else
      fprintf(stderr,"%s: wrong argument to AcceptDesktopGeometry\n",MyName);
  }
  else if  (StrEquals(resource,"ActivateWindowCmd")) {
    if (NetActivateWindow != NULL) {
      free(NetActivateWindow);
      NetActivateWindow  = NULL;
    }
    if (tline2 != NULL && strlen(tline2) > 0)
      CopyString(&NetActivateWindow,tline2);
    else
      CopyString(&NetActivateWindow,DEFAULT_NET_ACTIVATE_WINDOW_CMD);
  }
  else if  (StrEquals(resource,"CloseWindowCmd")) {
    if (NetCloseWindow != NULL) {
      free(NetCloseWindow);
      NetCloseWindow = NULL;
    }
    if (tline2 != NULL && strlen(tline2) > 0)
      CopyString(&NetCloseWindow,tline2);
    else
      CopyString(&NetCloseWindow,DEFAULT_NET_CLOSE_WINDOW_CMD);
  }
  else if  (StrEquals(resource,"UseWorkingAreaMaximizing")) {
    tline2 = GetNextToken(tline2, &arg1);
    if (StrEquals(arg1,"True"))
      UseStrutMaximize = True;
    else if (StrEquals(arg1,"False"))
      UseStrutMaximize = False;
    else
      fprintf(stderr,"%s: wrong argument to UseWorkingAreaMaximizing\n",MyName); 
  }
  else if  (StrEquals(resource,"UseDynamicMaximizing")) {
    tline2 = GetNextToken(tline2, &arg1);
    if (StrEquals(arg1,"True")) {
      UseDynamicStrutMaximize = True;
      UseStrutMaximize = True;
    }
    else if (StrEquals(arg1,"False")) {
      UseDynamicStrutMaximize = False;
    }
    else
      fprintf(stderr,"%s: wrong argument to UseDynamicMaximizing\n",MyName); 
  }
  else if  (StrEquals(resource,"StrutOverride")) {
    tline2 = GetNextToken(tline2, &arg1);
    if (StrEquals(arg1,"True")) {
      if (StrutOverride != True) {
	StrutOverride = True;
	update = 1;
      }
    }
    else if (StrEquals(arg1,"False")) {
      if (StrutOverride != False) {
	StrutOverride = False;
	update = 1;
      }
    }
    else
      fprintf(stderr,"%s: wrong argument to StrutOverride\n",MyName);
    if (Started && update) {
      NET_SetDesktopWorkArea();
      if (UseDynamicStrutMaximize)
	NET_UpdateMaximizedWindows();
    }
  }
  else if (StrEquals(resource, "DefaultMaximizePercentage"))
  {
    int val[2] = {100,100};

    if (GetIntegerArguments(tline2, NULL, val, 2) == 2)
    {
      if (val[0] > 0)
	 DefaultHorizMaximize = val[0];
      if (val[1] > 0)
	 DefaultVertMaximize= val[1];
    }
    else
      fprintf(stderr,"%s: wrong argument to DefaultMaximizePercentage\n",MyName);
  }
  else if  (StrEquals(resource,"UseWorkingAreaPlacement")) {
    tline2 = GetNextToken(tline2, &arg1);
    if (StrEquals(arg1,"True"))
      UseStrutPlacement = True;
    else if (StrEquals(arg1,"False"))
      UseStrutPlacement = False;
    else
      fprintf(stderr,"%s: wrong argument to UseWorkingAreaPlacement\n",MyName); 
  }
  else if  (StrEquals(resource,"SendMiniIcon")) {
    tline2 = GetNextToken(tline2, &arg1);
    if (StrEquals(arg1,"True"))
      DoSendMiniIcon = DO_SEND_NET_ICON;
    else if (StrEquals(arg1,"All"))
      DoSendMiniIcon = DO_SEND_NET_ICON_ALL;
    else if (StrEquals(arg1,"False"))
      DoSendMiniIcon = DO_NOT_SEND_NET_ICON;
    else {
      fprintf(stderr,"%s: wrong argument to SendMiniIcon\n",MyName); 
      update = 0;
    }
  }
  else if (StrEquals(resource, "MiniIconSize"))
  {
    int val[2] = {0,0};

    if (GetIntegerArguments(tline2, NULL, val, 2) == 2)
    {
      if (val[0] >= 0)
	MiniIconSize[0] = val[0];
      if (val[1] >= 0)
	MiniIconSize[1] = val[1];
      if (Started)
	UpdateMiniIconSize();
    }
    else
      fprintf(stderr,"%s: wrong argument to MiniIconSize\n",MyName);
  }
  /* Better has style */
  else if (StrEquals(resource, "MiniIconOverride"))
  {
    tline2 = GetNextToken(tline2, &arg1);
    if (arg1 == NULL || strlen(arg1) < 1)
      MiniIconOverrideMode = FVWM_NET_SET_ICON_MODE_ADD;
    else if (StrEquals(arg1, "no"))
      MiniIconOverrideMode = FVWM_NET_SET_ICON_MODE_NO;
    else if (StrEquals(arg1, "add"))
      MiniIconOverrideMode = FVWM_NET_SET_ICON_MODE_ADD;
    else if (StrEquals(arg1, "size"))
      MiniIconOverrideMode = FVWM_NET_SET_ICON_MODE_SIZE_OVERRIDE;
    else if (StrEquals(arg1, "all"))
      MiniIconOverrideMode = FVWM_NET_SET_ICON_MODE_OVERRIDE;
    else
      fprintf(stderr,"%s: wrong argument to MiniIconOverride\n",MyName);
  }
  else if (StrEquals(resource, "IconOverride"))
  {
    tline2 = GetNextToken(tline2, &arg1);
    if (arg1 == NULL || strlen(arg1) < 1)
      IconOverrideMode = FVWM_NET_SET_ICON_MODE_ADD;
    else if (StrEquals(arg1, "no"))
      IconOverrideMode = FVWM_NET_SET_ICON_MODE_NO;
    else if (StrEquals(arg1, "add"))
      IconOverrideMode = FVWM_NET_SET_ICON_MODE_ADD;
    else if (StrEquals(arg1, "size"))
      IconOverrideMode = FVWM_NET_SET_ICON_MODE_SIZE_OVERRIDE;
    else if (StrEquals(arg1, "all"))
      IconOverrideMode = FVWM_NET_SET_ICON_MODE_OVERRIDE;
    else
      fprintf(stderr,"%s: wrong argument to IconOverride\n",MyName);
  }
  else if  (StrEquals(resource,"MiniIconMaxSize")) {
    int val[1] = {0};
    if (GetIntegerArguments(tline2, NULL, val, 1) == 1 && val[0] > 0) {
        MiniIconMaxSize = val[0];
    }
    else
      fprintf(stderr,"%s: wrong argument to MaxMiniIconSize\n",MyName);
  }
  else if  (StrEquals(resource,"MyStrut")) {
    int val[4] = {0, 0, 0, 0};
    if (GetIntegerArguments(tline2, NULL, val, 4) == 4) {
      if (val[0] >= 0 && val[1] >= 0 && val[2] >= 0 && val[3] >= 0) {
	Scr.my_strut.left = val[0];
	Scr.my_strut.right = val[1];
	Scr.my_strut.top = val[2];
	Scr.my_strut.bottom = val[3];
	if (Started) {
	  NET_SetDesktopWorkArea();
	  if (UseDynamicStrutMaximize)
	    NET_UpdateMaximizedWindows();
	}
      }
      else
	fprintf(stderr,"%s: Arguments to MyStrut should 4 integers >= 0\n",MyName);
    }
    else
      fprintf(stderr,"%s: wrong arguments to MyStrut\n",MyName);
  }
  else
    fprintf(stderr,"%s: Unkown Configuration Option %s\n",MyName,resource);

  free(resource);
  if (arg1)
    free(arg1);
}

/* ***************************************************************************
 *
 * *************************************************************************** */
void SetNetStyle(net_style *style,net_style st)
{
  style->is_sticky = st.is_sticky;
  style->do_circulate_skip = st.do_circulate_skip;
  style->do_window_list_skip = st.do_window_list_skip;
  style->never_focus = st.never_focus;
  style->has_border = st.has_border;
  style->has_title = st.has_title;
  style->force_decoration = st.force_decoration;
  style->layer = st.layer;
}

/* ***************************************************************************
 *
 * *************************************************************************** */
void ParseNetStyle(char *line, net_style *st)
{
  char *option;
  char *token;
  char *rest;

  if (line == NULL)
    return;

  while (isspace((unsigned char)*line))
    line++;

  while (line && *line && *line != '\n')
  {
    line = GetNextFullOption(line, &option);
    if (!option)
      break;
    token = PeekToken(option, &rest);
    if (!token)
    {
      free(option);
      break;
    }

    if (StrEquals(token, "Sticky"))
      st->is_sticky = True;
    else if (StrEquals(token, "Slippery"))
      st->is_sticky = False;
    else if (StrEquals(token, "CirculateSkip"))
      st->do_circulate_skip = True;
    else if (StrEquals(token, "CirculateHit"))
      st->do_circulate_skip = False;
    else if (StrEquals(token, "WindowListSkip"))
      st->do_window_list_skip = True;
    else if (StrEquals(token, "WindowListHit"))
      st->do_window_list_skip = False;
    else if (StrEquals(token, "NoFocus"))
      st->never_focus = True;
    else if (StrEquals(token, "Focus"))
      st->never_focus = False;
    else if (StrEquals(token, "Title"))
      st->has_title = True;
    else if (StrEquals(token, "NoTitle"))
      st->has_title = False;
    else if (StrEquals(token, "Border"))
      st->has_border = True;
    else if (StrEquals(token, "NoBorder"))
      st->has_border = False;
    else if (StrEquals(token, "DecorationOverride"))
      st->force_decoration = True;
    else if (StrEquals(token, "NoOverride"))
      st->force_decoration = False;
    else if (StrEquals(token, "Layer")) {
      int val[1] = {-2};

      if (GetIntegerArguments(rest, NULL, val, 1) == 1) {
	if(val[0] < -1)
	  fprintf(stderr, "%s: NetStyle Layer argument must be > -2\n",MyName);
	else
	  st->layer = val[0];
      }
      else
	fprintf(stderr, "%s: NetStyle Layer need an integer as argument\n",
		MyName);
    }
    else
      fprintf(stderr, "%s: bad NetStyle command %s",MyName, option);
    free(option);
  }
}

/* ***************************************************************************
 *
 * *************************************************************************** */
void SavePersistentConfig(void)
{
  unsigned int size = 0;
  unsigned char *val;
  unsigned i,d;
  FILE *config_file;
  char filename[100];                   /* more than enough room */     

  val = NET_GetDesktopNames(&size);

  sprintf(filename,"%s/%s",getenv("FVWM_USERDIR"),FNH_PERSITENT_CONFIG_FILE);
  config_file = fopen(filename,"w");
  if (config_file == NULL) {
    fprintf(stderr,
            "%s: Open config file <%s> for write failed.\n",
            MyName, filename);
    return;
  }
  fprintf(config_file,"# This file was created by %s\n\n",MyName);

  fprintf(config_file,
	  "# Names of the desktop in utf8 encoding; you need an utf8 editor\n");
  fprintf(config_file,
	  "# for non ASCII characters\n");
  d=0;
  fprintf(config_file,"DesktopName 0 \""); 
  for(i=0;i<size && val != NULL;i=i+1) {
    if (val[i] == '\0') {
      d++;
      fprintf(config_file,"\"\nDesktopName %i \"",d);
    }
    else
      fprintf(config_file,"%c",val[i]);
  }
  if (val != NULL)
    free(val);
  fprintf(config_file,"\"\n");
  fprintf(config_file,"\n# Number of desktops\n");
  fprintf(config_file,"NumberOfDesktops %li\n",NumberOfDesktops);
  fclose(config_file);
}

/* ***************************************************************************
 *
 * *************************************************************************** */
void LoadPersistentConfig(void)
{
  FILE *config_file;
  char filename[100];
  char line[1024];
  char *tline;
  unsigned char *desktop_names[MAX_NUMBER_OF_DESKTOPS];
  char *token;
  char *next;
  int maxd = -1;
  int lenght = 0;

  sprintf(filename,"%s/%s",getenv("FVWM_USERDIR"),FNH_PERSITENT_CONFIG_FILE);
  config_file = fopen(filename,"r");
  if (config_file == NULL)
    return;

  tline = fgets(line, (sizeof line) - 1, config_file);
  while(tline)
  {
    int l;
    while(tline && (l = strlen(line)) < sizeof(line) && l >= 2 &&
	  line[l-2]=='\\' && line[l-1]=='\n')
    {
      tline = fgets(line+l-2,sizeof(line)-l+1, config_file);
    }
    tline=line;
    while(isspace((unsigned char)*tline))
      tline++;

    token = PeekToken(tline, &next);

    if (StrEquals(token, "DesktopName")) {
      int d = -1;
      char *arg;

      token = PeekToken(next, &next);
      if (token)
	sscanf(token, "%d", &d);
      if (d >= 0 && d < MAX_NUMBER_OF_DESKTOPS) {
	arg = PeekToken(next, &next);
	if (arg) {
	  desktop_names[d] = safestrdup(arg);
	  maxd = max(d,maxd);
	  lenght += strlen(desktop_names[d]) + 1;
	}
      }
    }
    else if (StrEquals(token, "NumberOfDesktops")) {
      int d = -1;

      token = PeekToken(next, &next);
      if (token)
	sscanf(token, "%d", &d);
      if (d > 0 && d <= MAX_NUMBER_OF_DESKTOPS && UsePNumberOfDesktops) {
	NumberOfDesktops = d;
	if (Started)
	  NET_SetNumberOfDesktops();
      }
    }

    tline = fgets(line,(sizeof line)-1, config_file);
  }

  if (maxd >= 0) {
    unsigned char *val;
    int j = 0;
    int k,i;

    val = (unsigned char *)safemalloc(lenght+maxd+1);
    for(i=0;i<=maxd;i++) {
      k=0;
      while(desktop_names[i] != NULL && desktop_names[i][k]) {
	val[j++]=desktop_names[i][k];
	k++;
      }
      val[j++] = '\0';
    }

    NET_SetDesktopNames(val,j);
    for(i=0;i<=maxd;i++)
      if (desktop_names[i] != NULL)
	free(desktop_names[i]);
    if (val != NULL)
      free(val);
  }

}

void ReadStartStopFiles(char *start_stop, int is_desktop)
{
  char cmd[238];
  
  if (is_desktop) {
    if (FnhSysDesktopStartStopFile != NULL) {
      sprintf(cmd,"Read %s-%s Quiet",FnhSysDesktopStartStopFile,start_stop);
      SendText(fd,cmd,None);
    }
  } else if (FnhSysStartStopFile != NULL) {
    sprintf(cmd,"Read %s-%s Quiet",FnhSysStartStopFile,start_stop);
    SendText(fd,cmd,None);
  }

  sprintf(cmd,"Read %s/%s-%s Quiet",
	  getenv("FVWM_USERDIR"),
	  (is_desktop) ? FNH_DESKTOP_START_STOP_PREFIX : FNH_START_STOP_PREFIX,
	  start_stop);
  SendText(fd,cmd,None);
}

