/*
 * Copyright 2001, Olivier Chapuis. No guarantees or warantees or anything
 * are provided or implied in any way whatsoever. Use this program at your
 * own risk. Permission to use this program for any purpose is given,
 * as long as the copyright is kept intact. 
 */

/* This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* from the glib code. Thanks! */

#include "config.h"

#if HAVE_ICONV
#include <stdio.h>
#include <stdlib.h>
#include <iconv.h>
#include <errno.h>
#include <string.h>
#ifdef HAVE_CODESET
#include <locale.h>
#include <langinfo.h>
#endif

#include <libs/fvwmlib.h>
#include <fvwm/fvwm.h>

#if defined(USE_LIBICONV) && !defined (_LIBICONV_H)
#error libiconv in use but included iconv.h not from libiconv
#endif
#if !defined(USE_LIBICONV) && defined (_LIBICONV_H)
#error libiconv not in use but included iconv.h is from libiconv
#endif

#include "utf8.h"

extern char *MyName;
size_t my_iconv(iconv_t cd, const char **inbuf, size_t *inbytes_left,
		char  **outbuf, size_t  *outbytes_left);


size_t my_iconv(iconv_t cd, const char **inbuf, size_t *inbytes_left,
		char  **outbuf, size_t  *outbytes_left)
{
  return iconv (cd, inbuf, inbytes_left, outbuf, outbytes_left);
}

char* UTF8_GetCharset()
{
  char *charset = NULL;

  charset = getenv("CHARSET");

  if (charset)
      return charset;

#if HAVE_CODESET     
  /* Get the name of the current locale.  */
  if ((setlocale(LC_CTYPE, "")) == NULL)
      fprintf(stderr,"%s: Can't set locale. Check your $LC_CTYPE or $LANG.\n",
	      MyName);
  charset = nl_langinfo(CODESET);
#endif
  return charset;
}

char* UTF8_charset2utf8(const char *charset, char *in)
{
  iconv_t cd;
  int have_error = 0;
  size_t nconv;
  size_t insize,outbuf_size,outbytes_remaining,len;
  const char *inptr;
  char *dest;
  char *outp;

  if (in == NULL)
    return NULL;

  inptr = in;

  cd = iconv_open ("UTF8", charset);
  if (cd == (iconv_t) -1)
  {
    /* Something went wrong.  */
    if (errno == EINVAL)
      fprintf(stderr, "%s: conversion from `%s' to `UTF8' not available\n",
	      MyName,charset);
    else
      fprintf (stderr, "%s: conversion from `%s' to `UTF8' fail (init)\n",
	       MyName,charset);
    
    /* Terminate the output string.  */
    return NULL;
  }

  len = strlen(in);
  outbuf_size = len + 1;
  outbytes_remaining = outbuf_size - 1;
  insize = len;
  outp = dest = safemalloc(outbuf_size);

 again:

  nconv = my_iconv(cd,(const char **)&inptr,&insize,&outp,&outbytes_remaining);
  if (nconv == (size_t) -1)
    {
      switch (errno)
	{
	case EINVAL:
	  /* Incomplete text, do not report an error */
	  break;
	case E2BIG:
	  {
	    size_t used = outp - dest;

	    outbuf_size *= 2;
	    dest = realloc (dest, outbuf_size);
		
	    outp = dest + used;
	    outbytes_remaining = outbuf_size - used - 1; /* -1 for nul */

	    goto again;
	  }
	default:
	  fprintf(stderr, "%s: Error during conversion from %s to UTF8: %s\n",
		  MyName,charset,strerror (errno));
	  have_error = True;
	  break;
	}
    }


  /* Terminate the output string.  */
  *outp = '\0';
  
  if (iconv_close (cd) != 0)
    fprintf (stderr, "%s: iconv_close fail", MyName);

  if (have_error)
    {
      free (dest);
      return NULL;
    }
  else
    return dest;
}
#endif
